;;;
;;;  grammar.scm: grammar handling functions
;;;
;;;
;;;  Copyright (C) 2001 Samuel Hocevar
;;;  $Id: grammar.scm,v 1.2 2001/11/27 18:24:57 sam Exp $
;;; 
;;;  Authors: Samuel Hocevar <sam@zoy.org>
;;; 
;;;  This program is free software; you can redistribute it and/or modify
;;;  it under the terms of the GNU General Public License as published by
;;;  the Free Software Foundation; either version 2 of the License, or
;;;  (at your option) any later version.
;;;  
;;;  This program is distributed in the hope that it will be useful,
;;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;  GNU General Public License for more details.
;;; 
;;;  You should have received a copy of the GNU General Public License
;;;  along with this program; if not, write to the Free Software
;;;  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.


;;;
;;;  get-grammar-metanames: get the list of metanames used in the grammar g
;;;

(define get-grammar-metanames (lambda (g)
  (if (null? g)
    ()
    (cons (caar g) (get-grammar-metanames (cdr g))))))

(define print-grammar-info (lambda (g)
  (display "Grammar information\n")
  (display "-------------------\n")
  ;(display "Grammar: ") (display g) (newline) (newline)
  (display "Metanames: ") (display (get-grammar-metanames g)) (newline)
  (display "Lexems: ") (display (get-grammar-lexems g)) (newline)))

;;;
;;;  get-grammar-lexems: get the list of lexems used in the grammar g
;;;

(define get-grammar-lexems (lambda (g)
  (if (null? g)
    ()
    (merge (get-production-lexems (car g)) (get-grammar-lexems (cdr g))))))

(define get-production-lexems (lambda (p)
  (if (null? p)
    ()
    (get-ruleset-lexems (cdr p)))))

(define get-ruleset-lexems (lambda (s)
  (if (null? s)
    ()
    (merge (get-rule-lexems (car s)) (get-ruleset-lexems (cdr s))))))

(define get-rule-lexems (lambda (r)
  (if (null? r)
    ()
    (merge (get-item-lexems (car r)) (get-rule-lexems (cdr r))))))

(define get-item-lexems (lambda (i)
  (cond
    ((symbol? i) ())
    ((string? i) (list i))
    ((list? i) (get-ruleset-lexems (cdr i))))))

;;;
;;;  get-symbol-tete: get tete(s) where s is a symbol of grammar g
;;;

(define get-symbol-tete (lambda (s g)
  (let ((p (assoc s g)))
    (if (not p) () (if (null? p) () (get-production-tete (cdr p) g))))))

(define get-production-tete (lambda (p g)
  (if (null? p)
    ()
    (merge (get-rule-tete (car p) g) (get-production-tete (cdr p) g)))))

(define get-rule-tete (lambda (r g)
  (if (null? r)
    () ; FIXME
    (if (list? (car r))
      (cond
        ((equal? '* (caar r)) (merge (get-production-tete (cdar r) g) (get-rule-tete (cdr r) g)))
        ((equal? '+ (caar r)) (get-production-tete (cdar r) g)))
      (get-item-tete (car r) g)))))

(define get-item-tete (lambda (i g)
  (cond
    ((string? i) (list i))
    ((symbol? i) (get-symbol-tete i g))
    (else ()))))

;;;
;;;  has-ruleset-only-lexems?: is a production made of terminals only?
;;;
;;;  This function is to be applied on a complete production such as
;;;  (foo ("a") ("b") ("c")), it returns #t if all the productions are
;;;  lexems and there is at least one, #f instead.

(define has-ruleset-only-lexems? (lambda (s)
  (if (null? s) #f (has-ruleset-only-lexems?/in s))))

(define has-ruleset-only-lexems?/in (lambda (s)
  (and (not (null? (car s)))
       (string? (caar s))
       (null? (cdar s))
       (or (null? (cdr s))
           (has-ruleset-only-lexems?/in (cdr s))))))

(define has-ruleset-only-characters? (lambda (s)
  (if (null? s) #f (has-ruleset-only-characters?/in s))))

(define has-ruleset-only-characters?/in (lambda (s)
  (and (not (null? (car s)))
       (string? (caar s))
       (null? (cdar s))
       (= 1 (string-length (caar s)))
       (or (null? (cdr s))
           (has-ruleset-only-characters?/in (cdr s))))))

;;;
;;;  Following functions are local
;;;

(define merge (lambda (l1 l2)
  (let ((insert (lambda (x l) (if (member x l) l (cons x l)))))
    (if (null? l1) l2 (insert (car l1) (merge (cdr l1) l2))))))

