;;;
;;;  lexer.scm: lexer generation functions
;;;
;;;
;;;  Copyright (C) 2001 Samuel Hocevar
;;;  $Id: lexer.scm,v 1.3 2001/11/27 18:24:57 sam Exp $
;;; 
;;;  Authors: Samuel Hocevar <sam@zoy.org>
;;; 
;;;  This program is free software; you can redistribute it and/or modify
;;;  it under the terms of the GNU General Public License as published by
;;;  the Free Software Foundation; either version 2 of the License, or
;;;  (at your option) any later version.
;;;  
;;;  This program is distributed in the hope that it will be useful,
;;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;  GNU General Public License for more details.
;;; 
;;;  You should have received a copy of the GNU General Public License
;;;  along with this program; if not, write to the Free Software
;;;  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  lexer: lex file f using automaton a

(define lexer (lambda (f a)
  (retourne (lex-port () (open-input-file f) () () () a 0 0))))

; chars port source tok lasttok automaton cell lastcell
(define lex-port (lambda (lch p s t lt a i li)
  (if (null? lch)
    (lex-port/in (read-char p) () p s t lt a i li)
    (lex-port/in (car lch) (cdr lch) p s t lt a i li))))

(define lex-port/in (lambda (ch lch p s t lt a i li)
  (let ((current-cell (assoc i a)))
    (if (eof-object? ch)
      (begin
        (close-input-port p)
        (if (null? lt)
          (if (cadr current-cell)
            (add-token t s)
            (begin (dtc-error "lexer stuck before EOF") #f))
          (add-token lt s)))
      (let ((next-i (get-next-cell (cddr current-cell) ch)))
        (if next-i
          (if (cadr current-cell)
            (lex-port lch p s (cons ch t) t a next-i i)
            (lex-port lch p s (cons ch t) lt a next-i li))
          (if (null? lt)
            (if (cadr current-cell)
              (lex-port (cons ch lch) p (add-token t s) () () a li 0)
              (begin
                (dtc-error "lexer stuck before  " (string ch) " ")
                (close-input-port p)
                #f))
            (if (cadr current-cell)
              (lex-port (list ch) p (add-token t s) () () a li 0)
              (lex-port (cons ch (add-remaining lch t lt)) p (add-token lt s) () () a li 0)))))))))

(define add-remaining (lambda (charlist bigtoken smalltoken)
  (if (equal? bigtoken smalltoken)
    charlist
    (cons (car bigtoken) (add-remaining charlist (cdr bigtoken) smalltoken)))))

(define add-token (lambda (t s)
  (if (or (null? t) (member (car t) '(#\space #\tab #\newline)))
    s
    (cons (build-token t) s))))

(define build-token (lambda (l)
  (if (null? l)
    ""
    (string-append (build-token (cdr l)) (string (car l))))))

(define get-next-cell (lambda (rule ch)
  (if (null? rule)
    #f
    (if (member ch (caar rule))
      (cadar rule)
      (get-next-cell (cdr rule) ch)))))

