;;;
;;;  system.scm: system functions
;;;
;;;
;;;  Copyright (C) 2001 Samuel Hocevar
;;;  $Id: system.scm,v 1.3 2001/11/28 00:55:02 sam Exp $
;;; 
;;;  Authors: Samuel Hocevar <sam@zoy.org>
;;; 
;;;  This program is free software; you can redistribute it and/or modify
;;;  it under the terms of the GNU General Public License as published by
;;;  the Free Software Foundation; either version 2 of the License, or
;;;  (at your option) any later version.
;;;  
;;;  This program is distributed in the hope that it will be useful,
;;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;  GNU General Public License for more details.
;;; 
;;;  You should have received a copy of the GNU General Public License
;;;  along with this program; if not, write to the Free Software
;;;  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.


;;;
;;;  parse-command-line: check for known options and set appropriate variables
;;;

(define parse-commandline (lambda ()
  (parse-commandline/in
    (if (defined? 'command-line) (cdr (command-line)) (cddr *argv*)))))

(define parse-commandline/in (lambda (l)
    (cond
      ((null? l))
       ; do nothing
      ((equal? "-v" (car l))
       (if *verbose*
         (set! *super-verbose* #t)
         (set! *verbose* #t))
       (parse-commandline/in (cdr l)))
      ((equal? "-vv" (car l))
       (set! *verbose* #t)
       (set! *super-verbose* #t)
       (parse-commandline/in (cdr l)))
      ((equal? "-l" (car l))
       (if (pair? (cdr l))
         (begin
           (set! *language* (cadr l))
           (parse-commandline/in (cddr l)))))
      (else
       (set! *source* (car l))
       (parse-commandline/in (cdr l))))))


;;;
;;;  dtc-info, dtc-error, dtc-fatal: combinations of display and string-append
;;;

(define dtc-info (lambda l
  (if *verbose* (display (string-append "dtc: " (list-append l) "\n")))))

(define dtc-error (lambda l
  (display (string-append "dtc error: " (list-append l) "\n"))))

(define dtc-fatal (lambda l
  (begin
    (display (string-append "dtc fatal error: " (list-append l) "\n"))
    (quit))))

(define list-append (lambda (l)
  (if (null? l) "" (string-append (car l) (list-append (cdr l))))))

